// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "BaseItem.generated.h"

USTRUCT(BlueprintType)
struct FItemSize
{
	GENERATED_BODY()

	int X;
	int Y;

	FItemSize(const int _X, const int _Y)
	{
		X = _X;
		Y = _Y;
	}

	FItemSize()
	{
		X = 1;
		Y = 1;
	}
};

UENUM(BlueprintType)
enum EItemRotation
{
	Horizontal UMETA(DisplayName = "Up"),
	Vertical UMETA(DisplayName = "Down")
};

/**
 * 
 */
UCLASS(Abstract)
class ENDLESSVENDETTA_API UBaseItem : public UObject
{
	GENERATED_BODY()

public:
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	int ItemID;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	FName ItemName;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	FText Description;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	FItemSize ItemSize = FItemSize();
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Instanced, Category = "Item")
	UTexture2D* ItemTexture;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Instanced, Category = "Item")
	UTexture2D* ItemTextureRotated;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	TEnumAsByte<EItemRotation> CurrentItemRotation = Horizontal;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	TSubclassOf<AActor> ItemActor;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Item")
	bool bIsRotated = false;
};
