﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "InventoryComponent.h"

#include "BaseItem.h"


// Sets default values for this component's properties
UInventoryComponent::UInventoryComponent()
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;

	// ...
}


// Called when the game starts
void UInventoryComponent::BeginPlay()
{
	Super::BeginPlay();

	// ...
}


// Called every frame
void UInventoryComponent::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	// ...
}

bool UInventoryComponent::AddItem(UBaseItem* Item)
{
	return false;
}

void UInventoryComponent::LineTraceForItemCheck()
{
}

void UInventoryComponent::Pickup()
{
	//TODO: Add pickup logic and create pickup Actor
	if (!IsValid(ItemToPickup)) return;
}

bool UInventoryComponent::IsRoomAvailable(UBaseItem* Item, const int TopLeftIndex)
{
	for (int i = 0; i < IndexToTile(TopLeftIndex).X + (Item->ItemSize.X - 1); i++)
	{
		for (int j = 0; j < IndexToTile(TopLeftIndex).Y + (Item->ItemSize.Y - 1); j++)
		{
			FInventoryTile TileToCheck;
			TileToCheck.X = i;
			TileToCheck.Y = j;
			if (!IsTileValid(TileToCheck)) return false;
			const TTuple<UBaseItem*, bool> ItemAtIndex = GetItemAtIndex(TileToIndex(TileToCheck));
			if (ItemAtIndex.Get<1>()) return false;
			if (IsValid(ItemAtIndex.Get<0>())) return false;
		}
	}
	return true;
}

FInventoryTile UInventoryComponent::IndexToTile(const int Index) const
{
	FInventoryTile Tile;
	Tile.X = Index % Columns;
	Tile.Y = Index / Columns;
	return Tile;
}

TTuple<UBaseItem*, bool> UInventoryComponent::GetItemAtIndex(const int Index)
{
	if (!InventoryItems.IsValidIndex(Index)) return MakeTuple(nullptr, false);
	return MakeTuple(InventoryItems[Index], true);
}

int UInventoryComponent::TileToIndex(const FInventoryTile InventoryTile) const
{
	return InventoryTile.X + InventoryTile.Y * Columns;
}

void UInventoryComponent::AddItemAt(UBaseItem* Item, const int TopLeftIndex)
{
	for (int i = 0; i < IndexToTile(TopLeftIndex).X + (Item->ItemSize.X - 1); i++)
	{
		for (int j = 0; j < IndexToTile(TopLeftIndex).Y + (Item->ItemSize.Y - 1); j++)
		{
			FInventoryTile TileToCheck;
			TileToCheck.X = i;
			TileToCheck.Y = j;
			if (!IsTileValid(TileToCheck)) return;
			InventoryItems.Insert(Item, TileToIndex(TileToCheck));
		}
	}
	IsDirty = true;
}

TMap<UBaseItem*, FInventoryTile> UInventoryComponent::GetAllItems()
{
	TMap<UBaseItem*, FInventoryTile> Items;
	for (int i = 0; i < InventoryItems.Num(); i++)
	{
		UBaseItem* Item = InventoryItems[i];
		if (!IsValid(Item)) continue;
		if (!Items.Contains(Item)) continue;
		Items.Add(Item, IndexToTile(i));
	}
	return Items;
}

void UInventoryComponent::RemoveItem(UBaseItem* Item)
{
	if (!IsValid(Item)) return;
	for (int i = 0; i < InventoryItems.Num(); i++)
	{
		if (InventoryItems[i] == Item)
		{
			InventoryItems.RemoveAt(i);
			IsDirty = true;
		}
	}
}

void UInventoryComponent::SpawnItem(UBaseItem* Item, FVector Location, FRotator Rotation)
{
}

bool UInventoryComponent::IsTileValid(const FInventoryTile InventoryTile) const
{
	if (InventoryTile.X >= 0 && InventoryTile.Y >= 0 && InventoryTile.Y < Columns && InventoryTile.Y < Rows)
	{
		return true;
	}
	return false;
}
